package com.gjjhhh;

import androidx.appcompat.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

public class MainActivity extends AppCompatActivity {
    private EditText messageInput;
    private Button sendButton;
    private RecyclerView messageList; // 修改为 RecyclerView
    private List<String> messages = new ArrayList<>();
    private MessageAdapter adapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // 初始化界面组件
        messageInput = findViewById(R.id.message_input);
        sendButton = findViewById(R.id.send_button);
        messageList = findViewById(R.id.message_list);

        // 添加 RecyclerView 的依赖项到 build.gradle(Module)

        messageList.setLayoutManager(new LinearLayoutManager(this)); // 使用 LayoutManager 设置布局管理器

        messageList.setHasFixedSize(true);
        adapter = new MessageAdapter(messages); // 修改此处的参数
        messageList.setAdapter(adapter);

        // 设置发送按钮点击监听器
        sendButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                sendMessage(messageInput.getText().toString());
                messageInput.setText("");
            }
        });

        // 获取初始消息列表
        fetchMessages();
    }

    private void sendMessage(String message) {
        // 发送消息到服务器
        try {
            String encodedMessage = URLEncoder.encode(message, StandardCharsets.UTF_8.name());
            URL url = new URL("http://dula.fun/send_message.php");
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setRequestMethod("POST");
            connection.setDoOutput(true); // 启用输出流
            connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            OutputStreamWriter writer = new OutputStreamWriter(connection.getOutputStream());
            writer.write("content=" + encodedMessage); // 使用正确的参数名 content
            writer.flush();
            writer.close();
            int responseCode = connection.getResponseCode(); // 获取响应码以检查是否成功
            if (responseCode == HttpURLConnection.HTTP_OK) {
                // 读取响应并检查是否为成功响应
                BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
                String inputLine;
                StringBuilder response = new StringBuilder();
                while ((inputLine = in.readLine())!= null) {
                    response.append(inputLine);
                }
                in.close();
                // 如果响应是空的，则表示成功
                if (response.length() == 0) {
                    // 更新本地消息列表
                    messages.add("Me: " + message);
                    adapter.notifyDataSetChanged();
                    // 获取最新的15条消息
                    fetchLatestMessages(15);
                } else {
                    handleNetworkError("服务器响应错误");
                }
            } else {
                handleNetworkError("发送消息失败");
            }
        } catch (Exception e) {
            handleNetworkError("发送消息时发生错误");
        }
    }

    private void fetchMessages() {
        // 从服务器获取消息列表
        try {
            String response = sendGETRequest("http://dula.fun/get_messages.php?count=100"); // 修改请求的 URL
            // 解析响应数据并添加到消息列表中
            List<String> serverMessages = parseJsonResponse(response);
            for (String msg : serverMessages) {
                messages.add(msg);
            }
            adapter.notifyDataSetChanged();
            // 获取最新的15条消息
            fetchLatestMessages(15);
        } catch (Exception e) {
            handleNetworkError("获取消息列表时发生错误");
        }
    }

    private void fetchLatestMessages(int count) {
        // 从服务器获取最新的count条消息
        try {
            String response = sendGETRequest("http://dula.fun/get_latest_messages.php?count=" + count);
            // 清除旧消息
            messages.clear();
            // 解析响应数据并添加到消息列表中
            List<String> latestMessages = parseJsonResponse(response);
            for (String msg : latestMessages) {
                messages.add(msg);
            }
            adapter.notifyDataSetChanged();
        } catch (Exception e) {
            handleNetworkError("获取最新消息时发生错误");
        }
    }

    private void handleNetworkError(String errorMessage) {
        Toast.makeText(this, errorMessage, Toast.LENGTH_LONG).show();
        // 在这里可以添加更多的错误处理逻辑
    }

    private String sendGETRequest(String urlStr) throws Exception {
        URL url = new URL(urlStr);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("GET");
        connection.setRequestProperty("User-Agent", "Java-HTTP-Client/1.0");
        int responseCode = connection.getResponseCode();
        if (responseCode == HttpURLConnection.HTTP_OK) {
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String response = "";
            String line;
            while ((line = in.readLine())!= null) {
                response += line;
            }
            in.close();
            return response;
        } else {
            throw new RuntimeException("Failed : HTTP error code : " + responseCode);
        }
    }

    private List<String> parseJsonResponse(String jsonResponse) {
        List<String> messages = new ArrayList<>();
        try {
            JSONObject jsonObject = new JSONObject(jsonResponse);
            JSONArray jsonArray = jsonObject.getJSONArray("messages");
            for (int i = 0; i < jsonArray.length(); i++) {
                JSONObject messageObject = jsonArray.getJSONObject(i);
                String content = messageObject.getString("content");
                messages.add(content);
            }
        } catch (JSONException e) {
            handleNetworkError("解析JSON响应时发生错误");
        }
        return messages;
    }
}
